@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <errno.h>
 
 #include "common/avl.h"
@@ -53,6 +57,7 @@
 #include "subsystems/oonf_rfc5444.h"
 #include "subsystems/oonf_telnet.h"
 #include "subsystems/oonf_timer.h"
+#include "subsystems/os_interface.h"
 
 #include "nhdp/nhdp_interfaces.h"
 
@@ -64,33 +69,71 @@
 #include "olsrv2/olsrv2_writer.h"
 
 /* definitions */
+
+/*! configuration option for locally attached networks */
 #define _LOCAL_ATTACHED_NETWORK_KEY "lan"
 
-#define LAN_DEFAULT_DOMAIN   0
-#define LAN_DEFAULT_METRIC   1
-#define LAN_DEFAULT_DISTANCE 2
+/**
+ * Default values for locally attached network parameters
+ */
+enum _lan_option_defaults {
+  LAN_DEFAULT_DOMAIN    = 0,//!< LAN_DEFAULT_DOMAIN
+  LAN_DEFAULT_METRIC    = 1,//!< LAN_DEFAULT_METRIC
+  LAN_DEFAULT_DISTANCE  = 2,//!< LAN_DEFAULT_DISTANCE
+};
 
+/*! locally attached network option for source-specific prefix */
 #define LAN_OPTION_SRC    "src="
+
+/*! locally attached network option for outgoing metric */
 #define LAN_OPTION_METRIC "metric="
+
+/*! locally attached network option for domain */
 #define LAN_OPTION_DOMAIN "domain="
+
+/*! locally attached network option for hopcount distance */
 #define LAN_OPTION_DIST   "dist="
 
+/**
+ * olsrv2 plugin config
+ */
 struct _config {
+  /*! topology control interval */
   uint64_t tc_interval;
+
+  /*! topology control validity */
   uint64_t tc_validity;
 
+  /*! olsrv2 f_hold_time */
   uint64_t f_hold_time;
-  uint64_t p_hold_time;
-  struct netaddr_acl routable;
 
-  /* configuration for originator set */
+  /*! olsrv2 p_hold_time */
+  uint64_t p_hold_time;
+
+  /*! decides NHDP routable status */
+  bool nhdp_routable;
+
+  /*! IP filter for routable addresses */
+  struct netaddr_acl routable_acl;
+
+  /*! IP filter for valid originator */
   struct netaddr_acl originator_acl;
 };
 
+/**
+ * Additional parameters of a single locally attached network
+ */
 struct _lan_data {
+  /*! domain of LAN */
   struct nhdp_domain *domain;
+
+  /*! source prefix */
   struct netaddr source_prefix;
+
+  /*! olsrv2 metric */
   uint32_t metric;
+
+  /*! routing metric (distance) */
   uint32_t dist;
 };
 
@@ -103,10 +146,10 @@ static void _cleanup(void);
 static const char *_parse_lan_parameters(struct os_route_key *prefix,
 		struct _lan_data *dst, const char *src);
 static void _parse_lan_array(struct cfg_named_section *section, bool add);
-static void _cb_generate_tc(void *);
+static void _cb_generate_tc(struct oonf_timer_instance *);
 
 static void _update_originator(int af_family);
-static int _cb_if_event(struct oonf_interface_listener *);
+static int _cb_if_event(struct os_interface_listener *);
 
 static void _cb_cfg_olsrv2_changed(void);
 static void _cb_cfg_domain_changed(void);
@@ -143,23 +186,27 @@ static struct cfg_schema_entry _olsrv2_entries[] = {
     "Holdtime for forwarding set information", 100),
   CFG_MAP_CLOCK_MIN(_config, p_hold_time, "processing_hold_time", "300.0",
     "Holdtime for processing set information", 100),
-  CFG_MAP_ACL_V46(_config, routable, "routable",
+  CFG_MAP_BOOL(_config, nhdp_routable, "nhdp_routable", "no",
+    "Decides if NHDP interface addresses"
+    " are routed to other nodes. 'true' means the 'routable_acl' parameter"
+    " will be matched to the addresses to decide."),
+  CFG_MAP_ACL_V46(_config, routable_acl, "routable_acl",
       OLSRV2_ROUTABLE_IPV4 OLSRV2_ROUTABLE_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter to decide which addresses are considered routable"),
 
   CFG_VALIDATE_LAN(_LOCAL_ATTACHED_NETWORK_KEY, "",
     "locally attached network, a combination of an"
     " ip address or prefix followed by an up to four optional parameters"
-    " which define link metric cost, hopcount distance and domain of the prefix"
-    " ( <"LAN_OPTION_METRIC"...> <"LAN_OPTION_DIST"...>"
-    " <"LAN_OPTION_DOMAIN"...> <"LAN_OPTION_SRC"...> ).",
+    " which define link metric cost, hopcount distance, domain of the prefix"
+    " and the source-prefix ( <"LAN_OPTION_METRIC"...> <"LAN_OPTION_DIST"...>"
+    " <"LAN_OPTION_DOMAIN"<num>/all> <"LAN_OPTION_SRC"...> ).",
     .list = true),
 
   CFG_MAP_ACL_V46(_config, originator_acl, "originator",
     OLSRV2_ORIGINATOR_IPV4 OLSRV2_ORIGINATOR_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter for router originator addresses (ipv4 and ipv6)"
     " from the interface addresses. Olsrv2 will prefer routable addresses"
-    " over linklocal addresses."),
+    " over linklocal addresses and addresses from loopback over other interfaces."),
 };
 
 static struct cfg_schema_section _olsrv2_section = {
@@ -172,9 +219,9 @@ static struct cfg_schema_section _olsrv2_section = {
 
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_RFC5444_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_NHDP_SUBSYSTEM,
 };
 static struct oonf_subsystem _olsrv2_subsystem = {
@@ -203,8 +250,9 @@ static struct oonf_timer_instance _tc_timer = {
 };
 
 /* global interface listener */
-struct oonf_interface_listener _if_listener = {
-  .process = _cb_if_event,
+static struct os_interface_listener _if_listener = {
+  .name = OS_INTERFACE_ANY,
+  .if_changed = _cb_if_event,
 };
 
 /* global variables */
@@ -239,18 +287,13 @@ _init(void) {
     return -1;
   }
 
-  _protocol = oonf_rfc5444_add_protocol(RFC5444_PROTOCOL, true);
-  if (_protocol == NULL) {
-    return -1;
-  }
-
+  _protocol = oonf_rfc5444_get_default_protocol();
   if (olsrv2_writer_init(_protocol)) {
-    oonf_rfc5444_remove_protocol(_protocol);
     return -1;
   }
 
   /* activate interface listener */
-  oonf_interface_add_listener(&_if_listener);
+  os_interface_add(&_if_listener);
 
   /* activate the rest of the olsrv2 protocol */
   olsrv2_lan_init();
@@ -281,10 +324,10 @@ _initiate_shutdown(void) {
 static void
 _cleanup(void) {
   /* remove interface listener */
-  oonf_interface_remove_listener(&_if_listener);
+  os_interface_remove(&_if_listener);
 
   /* cleanup configuration */
-  netaddr_acl_remove(&_olsrv2_config.routable);
+  netaddr_acl_remove(&_olsrv2_config.routable_acl);
   netaddr_acl_remove(&_olsrv2_config.originator_acl);
 
   /* cleanup all parts of olsrv2 */
@@ -294,7 +337,7 @@ _cleanup(void) {
   olsrv2_lan_cleanup();
 
   /* free protocol instance */
-  oonf_rfc5444_remove_protocol(_protocol);
+  _protocol = NULL;
 }
 
 /**
@@ -314,13 +357,25 @@ olsrv2_get_tc_validity(void) {
 }
 
 /**
- * @return acl for checking if an address is routable
+ * @param addr NHDP address to be checked
+ * @return true if address should be routed, false otherwise
  */
-const struct netaddr_acl *
-olsrv2_get_routable(void) {
-    return &_olsrv2_config.routable;
+bool
+olsrv2_is_nhdp_routable(struct netaddr *addr) {
+  if (!_olsrv2_config.nhdp_routable) {
+    return false;
+  }
+  return olsrv2_is_routable(addr);
 }
 
+/**
+ * @param addr address to be checked
+ * @return true if address should be routed, false otherwise
+ */
+bool
+olsrv2_is_routable(struct netaddr *addr) {
+  return netaddr_acl_check_accept(&_olsrv2_config.routable_acl, addr);
+}
 
 /**
  * default implementation for rfc5444 processing handling according
@@ -364,9 +419,10 @@ olsrv2_mpr_shall_process(
 /**
  * default implementation for rfc5444 forwarding handling according
  * to MPR settings.
- * @param context
- * @param vtime
- * @return
+ * @param context RFC5444 reader context
+ * @param source_address source address of RFC5444 message
+ * @param vtime validity time of message information
+ * @return true if message was forwarded, false otherwise
  */
 bool
 olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
@@ -522,8 +578,8 @@ olsrv2_validate_lan(const struct cfg_schema_entry *entry,
         "    - '"LAN_OPTION_METRIC"<m>' the link metric of the LAN (between %u and %u)."
         " The default is 0.", RFC7181_METRIC_MIN, RFC7181_METRIC_MAX);
     cfg_append_printable_line(out,
-        "    - '"LAN_OPTION_DOMAIN"<d>' the domain of the LAN (between 0 and 255)."
-        " The default is 0.");
+        "    - '"LAN_OPTION_DOMAIN"<d>' the domain of the LAN (between 0 and 255) or 'all'."
+        " The default is all.");
     cfg_append_printable_line(out,
         "    - '"LAN_OPTION_DIST"<d>' the hopcount distance of the LAN (between 0 and 255)."
         " The default is 2.");
@@ -578,7 +634,7 @@ _parse_lan_parameters(struct os_route_key *prefix,
   unsigned ext;
 
   ptr = src;
-  dst->domain = nhdp_domain_get_by_ext(LAN_DEFAULT_DOMAIN);
+  dst->domain = NULL;
   dst->metric = LAN_DEFAULT_METRIC;
   dst->dist   = LAN_DEFAULT_DISTANCE;
 
@@ -592,13 +648,19 @@ _parse_lan_parameters(struct os_route_key *prefix,
       }
     }
     else if (strncasecmp(buffer, LAN_OPTION_DOMAIN, 7) == 0) {
-      ext = strtoul(&buffer[7], NULL, 10);
-      if ((ext == 0 && errno != 0) || ext > 255) {
-        return "an illegal domain parameter";
+      fprintf(stderr, "domain='%s'\n", &buffer[7]);
+      if (strcasecmp(&buffer[7], "all") == 0) {
+        dst->domain = NULL;
       }
-      dst->domain = nhdp_domain_get_by_ext(ext);
-      if (dst->domain == NULL) {
-        return "an unknown domain extension number";
+      else {
+        ext = strtoul(&buffer[7], NULL, 10);
+        if ((ext == 0 && errno != 0) || ext > 255) {
+          return "an illegal domain parameter";
+        }
+        dst->domain = nhdp_domain_get_by_ext(ext);
+        if (dst->domain == NULL) {
+          return "an unknown domain extension number";
+        }
       }
     }
     else if (strncasecmp(buffer, LAN_OPTION_DIST, 5) == 0) {
@@ -661,7 +723,7 @@ _parse_lan_array(struct cfg_named_section *section, bool add) {
       continue;
     }
 
-    os_route_init_sourcespec_prefix(&prefix, &addr);
+    os_routing_init_sourcespec_prefix(&prefix, &addr);
 
     /* truncate address */
     netaddr_truncate(&prefix.dst, &prefix.dst);
@@ -670,27 +732,39 @@ _parse_lan_array(struct cfg_named_section *section, bool add) {
       continue;
     }
 
-    if (add) {
-      olsrv2_lan_add(data.domain, &prefix, data.metric, data.dist);
+    if (data.domain == NULL) {
+      list_for_each_element(nhdp_domain_get_list(), data.domain, _node) {
+        if (add) {
+          olsrv2_lan_add(data.domain, &prefix, data.metric, data.dist);
+        }
+        else {
+          olsrv2_lan_remove(data.domain, &prefix);
+        }
+      }
     }
     else {
-      olsrv2_lan_remove(data.domain, &prefix);
+      if (add) {
+        olsrv2_lan_add(data.domain, &prefix, data.metric, data.dist);
+      }
+      else {
+        olsrv2_lan_remove(data.domain, &prefix);
+      }
     }
   }
 }
 
 /**
  * Callback to trigger normal tc generation with timer
- * @param ptr
+ * @param ptr timer instance that fired
  */
 static void
-_cb_generate_tc(void *ptr __attribute__((unused))) {
+_cb_generate_tc(struct oonf_timer_instance *ptr __attribute__((unused))) {
   if (nhdp_domain_node_is_mpr() || !avl_is_empty(olsrv2_lan_get_tree())) {
     olsrv2_writer_send_tc();
   }
 }
 
-static int
+static uint32_t
 _get_addr_priority(const struct netaddr *addr) {
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf;
@@ -707,29 +781,29 @@ _get_addr_priority(const struct netaddr *addr) {
   if (netaddr_get_address_family(addr) == AF_INET) {
     if (netaddr_is_in_subnet(&NETADDR_IPV4_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (linklocal)",
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
           netaddr_to_string(&nbuf, addr));
-      return 2;
+      return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 4 (routable)",
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
         netaddr_to_string(&nbuf, addr));
-    return 4;
+    return 2;
   }
 
   if (netaddr_get_address_family(addr) == AF_INET6) {
     if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (linklocal)",
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
           netaddr_to_string(&nbuf, addr));
-      return 2;
+      return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 4 (routable)",
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
         netaddr_to_string(&nbuf, addr));
-    return 4;
+    return 2;
   }
 
   /* unknown */
@@ -746,13 +820,13 @@ _get_addr_priority(const struct netaddr *addr) {
 static void
 _update_originator(int af_family) {
   const struct netaddr *originator;
-  struct nhdp_interface *n_interf;
-  struct os_interface *interf;
+  struct nhdp_interface *nhdp_if;
+  struct os_interface_listener *if_listener;
   struct netaddr new_originator;
-  int new_priority;
-  int old_priority;
-  int priority;
-  size_t i;
+  struct os_interface_ip *ip;
+  uint32_t new_priority;
+  uint32_t old_priority;
+  uint32_t priority;
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str buf;
 #endif
@@ -767,26 +841,27 @@ _update_originator(int af_family) {
 
   netaddr_invalidate(&new_originator);
 
-  avl_for_each_element(nhdp_interface_get_tree(), n_interf, _node) {
-    interf = nhdp_interface_get_coreif(n_interf);
+  avl_for_each_element(nhdp_interface_get_tree(), nhdp_if, _node) {
+    if_listener = nhdp_interface_get_if_listener(nhdp_if);
 
     /* check if originator is still valid */
-    for (i=0; i<interf->data.addrcount; i++) {
-      struct netaddr *addr = &interf->data.addresses[i];
-
-      if (netaddr_get_address_family(addr) == af_family) {
-        priority = _get_addr_priority(addr);
+    avl_for_each_element(&if_listener->data->addresses, ip, _node) {
+      if (netaddr_get_address_family(&ip->address) == af_family) {
+        priority = _get_addr_priority(&ip->address) * 4;
         if (priority == 0) {
           /* not useful */
           continue;
         }
 
-        if (netaddr_cmp(originator, addr) == 0) {
+        if (if_listener->data->flags.loopback) {
+          priority += 2;
+        }
+        if (netaddr_cmp(originator, &ip->address) == 0) {
           old_priority = priority + 1;
         }
 
         if (priority > old_priority && priority > new_priority) {
-          memcpy(&new_originator, addr, sizeof(new_originator));
+          memcpy(&new_originator, &ip->address, sizeof(new_originator));
           new_priority = priority;
         }
       }
@@ -806,7 +881,7 @@ _update_originator(int af_family) {
  * @return always 0
  */
 static int
-_cb_if_event(struct oonf_interface_listener *listener __attribute__((unused))) {
+_cb_if_event(struct os_interface_listener *if_listener __attribute__((unused))) {
   _update_originator(AF_INET);
   _update_originator(AF_INET6);
   return 0;
