@@ -50,28 +50,60 @@
 #include "test_rfc5444_interop.h"
 
 static uint8_t _binary[] = {
-    0x08, 0x00, 0x1d, 0x01, 0x0f, 0x00, 0x06, 0x00, 0x00 };
+    0x0c, 0x00, 0x0d, 0x00, 0x02, 0x01, 0x00, 0x01, 0x03, 0x00, 0x08, 0x00,
+    0x02, 0x01, 0x00, 0x02, 0xf3, 0x00, 0x0e, 0x0a, 0x00, 0x00, 0x01, 0xff,
+    0x01, 0x30, 0x39, 0x00, 0x00 };
+
+static uint8_t _originator[] = { 10, 0, 0, 1 };
+
+static struct test_tlv _msgtlvs[] = {
+  { .type = 1 },
+};
 
 static struct test_message _msgs[] = {
   {
     .type = 1,
-    .addrlen = 16,
+    .addrlen = 4,
+
+    .tlv_count = ARRAYSIZE(_msgtlvs),
+    .tlvs = _msgtlvs,
+  },
+  {
+    .type = 2,
+    .addrlen = 4,
+    .flags = 240,
+
+    .has_originator = true,
+    .originator = _originator,
+    .has_hopcount = true,
+    .hopcount = 1,
+    .has_hoplimit = true,
+    .hoplimit = 255,
+    .has_seqno = true,
+    .seqno = 12345,
   },
 };
 
-static struct test_packet test29 = {
-  .test = "Interop 2010 Test 29",
+static struct test_tlv _pkttlvs[] = {
+  { .type = 1 },
+};
+
+static struct test_packet test13 = {
+  .test = "Interop 2010 Test 13",
   .binary = _binary,
   .binlen = ARRAYSIZE(_binary),
 
   .version = 0,
-  .flags = 0x08,
+  .flags = 0x0c,
 
   .has_seq = true,
-  .seqno = 29,
+  .seqno = 13,
+
+  .tlv_count = ARRAYSIZE(_pkttlvs),
+  .tlvs = _pkttlvs,
 
   .msg_count = ARRAYSIZE(_msgs),
   .msgs = _msgs,
 };
 
-ADD_TEST(test29)
+ADD_TEST(test13)
